__code__='Gas1]?ZXsi&Dr[^a)OI]gee!X<4!@>#NVb5%)Xb-*B/\\t-mhm%%HG<=\'^[2D,>7=dS2cu0.@oFO0>c?!A*B1r_%XG<eX2Z%kAKI1\\,6\'iY3pL`_qa]!bo^W]J!%Pd0EV"]"X(TR@!cY(8:VCn$G;c*%ak?Q@00V_%DX=K!alCT&[2^h!H8A7*Pc7%Mguh`G6,=+QN1(ehZLM"pcdU@J0+iA"Sa_0\'u6l#T)`1BQAa\'_k.:<A(&4\'(/Jp?12E:kRJS`-$$0(b&Yl3(\'Vo3HTI`eXM.#7+BF?V4oFuWj%=Vo/Eb,P^sCIAnn=WK.!N(j44F=\\`ZCClr@@#JWLAtK)dmZ;r4!LI/\\TE.f$Qjc;AjZe;3oo<0dW=:gA+M1ji$?#"*!0L6O-+6*Z_7uE8>7Vea*c9]Es2Zq<KfuC;X];oncPDL67>Z2hd`Q.6>Df45a8+T5fLc&FeY8T5EE"Ke=p899\\SLWuH7r?.Dmn2ckU=`\'PFXO"2`iXO/E>H)NW[ZE?-s7<[I%QD&[ZpM$-(EfbI6l#;k1e=?Ln\'XP9<%N;/M2k=QdmP)fD1^cto#daLjSS(7Y"bRAEN&_2c4j4W`<SD?k>)Kegarg6j.J:tt7&SDV\'3fseZOdF")WWm*NUEPlnbg4VKU$?tMaLaRb6d_nnG/Z2aI#V/TTj8T08+U)!_j;#0#WD\\pRfAplEmh^07s&QnZil5a;*_M]=Ai`Y\'-&G6d\\Sc`4osr"RW,I2aKj2NeI[]JNqJq%4l]$\'I6t\'^A(%)=;F=D0M0Nu23HhB:Vp!p7\\>$q/]2]ijlF-HodQ7<2M8nd7OIdq\\?k*\'j):;WLn-ck].P$n,md0]PYGRu0JFU)X8gF0XU5O7cW1FI:JJ%o*2eS,N#GCq<0a?`$,mJ84oi-Y$jkRM/`;3=&NT1!Bu[HNm3q-l3s%D$)I?a-8rM<^(F!Pg`@>>(,JB$_LEL@%TE@PF!\'hf"H?9t-L4Xn\\VOU(l).Y&`4f/\'2(.6E<_P,?0F?#"MW`:9.Xhr?.+,W^=-2[qnH9Nq6-gPi>NEbc7?9;bPg?mS%!FF*PFC-&P@?7:C@[Yp,XV^Iq",$i~>'



__code__='Gat=(s)6X[\'RuIZ=keq<FCd/YR+W<e83#oB\'g`0\\Z4nXrTj##2AK]1h]=)#dgH4<#WRUmr8c6G4`F$W3B7!4<`2\'j348\\I&kKC4#754=WMlr]9.fQ,lG@1RC$OT.Lf>-p`!@%VC^a\\ctj=cT(5l*>=*Pa\\N(>oA3H,\'U!7DOh3QN5nF]F9#gE!c#^^h(<^J1H-l!+#d*&F^p[l[U&ZIKF`rn0[(kJ@C?rb)7Tf4o4aSr\'B*JN93C6=FkQZcG=3.`65o(ptUG``9m\\U`_o=&iAM+uT9B^/-k%=t=lchZf?\\:)%YYRHT!W[^i2H8lf[O]QK_.6;4&_Y]Ku6i1FEJJAkAgkg&TI,;\'=7p+Befr#/:IcYW+VQ"Z?G0%!VsWZX5M-l$4s0gF%P"C(F]Q4MdYkmj8O5jdDg@!)9i<lHW#W&/&N#>@8]WX>?%r!^9*@$bINL/?!;46Sn-H!8I^"a(ON#Fj%e<?>;hjq\'N?8(>(p=eH#):%HDaS:Vjclf*@F:rlspEKq6<Z?Es=/WD2B(JZ<.a*D<U),^QcuY6\'-h2)$FR!5)_9S#sVb%O"X4p<9psm=Xkrk2@H[NOt7fp6#^_Tp-g`S_1@9FoS8\'Mfqd#D,4eKB5MU7fr20WU0&4fJi^fr#6oS2C37;]Q40rj4lhpWTkNM-N;g\'o&GJ#=CHZ"\\qpPh[Qmk@F/W8fFO1ZcbN*\\0R9ZMaJK*7Ifc;l9DG/TrqH9Wi.Sn\'(&8G:)W]]/L0@K`;lRPcA,n3m1bSM,qA_f_!GUZ\\Bra[g5V+>NUK:PNn6E2r2u4X(:s\\cHV?+p;94CAbp#r>c.&-rRLf;>@7@&S+X2XWhlV*q2FmbWKo"[=]T5)pTF=DDW3CS:Pi5E?WAj4h!2>$m.t=N`b8^K#B@=V=1hf1H^:ja$hZTo>EjMaGJgN+XBQ6[2iJE,\'u4b],H`t9#M$<Xbi0+CA5oXkh.L_nfYU+#BBF-@<OWqHRO2B^o6;EpDgV^gMpZ\\AiI94JhREhYgJBlE5L[ZQdti">l*S-<\'=5BK4EDJhk<<W33O$R`=h%tDd#"7;pSTjcp3b]QJfm`"&lrc/1fe@6``eB1]5V0<iL.tM"eKHmO)S.j6eS&[A<u4*IKAGCCI\'M-f)PYTDleaqj\'O0TU8\\~>'




"example plug in flowable for a bar chart in RML"

from reportlab.lib import colors
from reportlab.graphics.charts.barcharts import VerticalBarChart
from reportlab.graphics.charts.linecharts import HorizontalLineChart
from reportlab.graphics.shapes import Drawing

def sample1bar(data=[(13, 5, 20, 22, 37, 45, 19, 4),
                     (5, 20, 22, 37, 45, 19, 4, 13),
                     (20, 22, 37, 45, 19, 4, 13, 5)]):
    # sanity check:
    for d in data:
        if len(d)!=8:
            raise ValueError("bar chart data must have 8 elements "+repr(data))
    drawing = Drawing(400, 200)
    
    bc = VerticalBarChart()
    bc.x = 50
    bc.y = 50
    bc.height = 125
    bc.width = 300
    bc.data = data

    bc.strokeColor = colors.black

    bc.valueAxis.valueMin = 0
    bc.valueAxis.valueMax = 60
    bc.valueAxis.valueStep = 15
    
    bc.categoryAxis.labels.boxAnchor = 'ne'
    bc.categoryAxis.labels.dx = 8
    bc.categoryAxis.labels.dy = -2
    bc.categoryAxis.labels.angle = 30

    catNames = 'Jan Feb Mar Apr May Jun Jul Aug'.split()
    catNames = [n+'-99' for n in catNames]
    bc.categoryAxis.categoryNames = catNames
    drawing.add(bc)

    return drawing

def sample1line(datastring="""
	1997 1998 1999 2000 2001 ,
	pink -10 11 40 22 30 ,
	blue 11 40 22 30 -10,
	cyan 40 22 30 -10 11
"""):
    # parse the data
    lines = datastring.split(",")
    titlestring = lines[0]
    titles = titlestring.split()
    data = []
    alldata = []
    linecolors = []
    for dataline in lines[1:]:
        sline = dataline.split()
        linecolor = sline[0]
        numbers = list(map(float, sline[1:]))
        linecolors.append(linecolor)
        data.append(numbers)
        alldata.extend(numbers)
    drawing = Drawing(400, 200)
    
    bc = HorizontalLineChart()
    bc.x = 25
    bc.y = 50
    bc.height = 125
    bc.width = 300
    bc.data = data

    bc.strokeColor = colors.black

    bc.valueAxis.valueMin = min(0, min(alldata)-5) #-10
    bc.valueAxis.valueMax = max(20, max(alldata)+5) #40
    bc.valueAxis.valueStep = 10
    bc.valueAxis.visibleGrid = 1
    bc.valueAxis.gridStrokeColor = colors.blue
    bc.valueAxis.gridStart = 0
    bc.valueAxis.gridEnd = 300
    bc.valueAxis.labelTextFormat ="%s%%"
    #bc.valueAxis.joinAxisPos = -10
    
    bc.categoryAxis.labels.boxAnchor = 'ne'
    bc.categoryAxis.labels.dx = 8
    bc.categoryAxis.labels.dy = -2
    bc.categoryAxis.labels.angle = 0
    bc.categoryAxis.joinAxis = bc.valueAxis
    bc.categoryAxis.joinAxisMode = "bottom"
    bc.categoryAxis.visibleGrid = 1
    
    #bc.lines[0].strokeColor = colors.pink
    #bc.lines[1].strokeColor = colors.lavender
    #bc.lines[2].strokeColor = colors.skyblue
    linenumber = 0
    for colorname in linecolors:
        bc.lines[linenumber].strokeColor = getattr(colors, colorname)
        bc.lines[linenumber].strokeWidth = 3
        linenumber = linenumber+1

    catNames = titles
    bc.categoryAxis.categoryNames = catNames
    drawing.add(bc)

    return drawing

lineChartFlowable = sample1line

def chartFlowable(datastring):
    "convert the string, then make the chart"
    #return sample1line()
    datalist = datastring.split()
    data = list(map(float, datalist))
    alldata = [data]
    return sample1bar(alldata)

if __name__=="__main__":
    # just check to make sure code runs, don't save anything
    test = chartFlowable("13 5 20 22 37 45 19 4")
    print(test.getProperties(recur=1))



# Copyright ReportLab Europe Ltd. 2000-2016
# $Id$
from rlextra.rml2pdf import rml2pdf
import os, string, time
from rlextra.ers import webapp
from . import travel
import preppy
from rlextra.radxml.xmlutils import xml2doctree

class TravelPlanApp(webapp.WebApp):

    def __init__(self, **kw):
        self._setKeywords(**kw)
        webapp.WebApp.__init__(
            self,
            defaultAction='search',
            rsrcUrl='/rsrc/travelplan/',
            pageTemplate='travel_template.html'
            )
    def getRsrcUrl(self,name):
        return self.rsrcUrl+name
    
    def action_makepdf(self,request,response):
        started = time.time()
        params=request.params
        orig = params['orig']
        dest = params['dest']
        oDate = params['oYMD']
        rDate = params['rYMD']
        flightClass = params['flightClass']
        isRoundTrip = params['isRoundTrip']
        numOfAdults = params['numOfAdults']
        customerName = params['customerName']
        tripTitle = params['tripTitle']
        subtitle = params['subtitle']
        if isRoundTrip == 'True': isRT = True
        else: isRT = False
        presaleXML = travel.getPresaleXML(tripTitle,subtitle,customerName,orig, dest,oDate,rDate,flightClass,isRT)
        data=xml2doctree(presaleXML,0)
        module=preppy.getModule('template_presale.prep')
        rmlText=module.getOutput(dict(data=data))
        if int(getattr(self,'SAVE_LASTEST_RML',0)):
            f = open('save_lastest.rml','w')
            f.write(rmlText)
            f.close()
        outfile='presale.pdf'
        pdf=rml2pdf.go(rmlText, outfile, outDir=self.OUTDIR)
        location = '%s/%s' % (self.HTTP_OUTDIR,outfile)
        params=request.params
        finished = time.time()
        timeTaken = finished - started
        params['timeTaken'] = timeTaken
        params['location']= location
        self.runView(request,response,'download')

    def action_makeitinerarypdf(self,request,response):
        started = time.time()
        params=request.params
        ticket_id = params['ticket_id']
        roomBooking_id = params['roomBooking_id']
        numOfAdults = params['numOfAdults']
        customerName = params['customerName']
        tripTitle = params['tripTitle']
        subtitle = params['subtitle']
        itineraryXML = travel.getStep4XML(tripTitle,subtitle,customerName, ticket_id, roomBooking_id)
        data=xml2doctree(itineraryXML,0)
        module=preppy.getModule('template_itinerary.prep')
        rmlText=module.getOutput(dict(data=data))
        outfile='postsale.pdf'
        pdf=rml2pdf.go(rmlText, outfile, outDir=self.OUTDIR)
        location = '%s/%s' % (self.HTTP_OUTDIR,outfile)
        params=request.params
        finished = time.time()
        timeTaken = finished - started
        params['timeTaken'] = timeTaken
        params['location']=location
        self.runView(request,response,'download')



__code__='Gas1];/[CB&>$Qs4,t7`j+]moV+41P7cN2[&0Z]<Q0VJ5/gdUQ#.^DUj^s]]F2#9SXV9l^5,AlFr"8o45N?)G(HaEnnMR14Z$/_(h3m`$)=P%a_qaDZc(AZ]J!%QO0EQKP!YF9IYWX^K,X;\\r!FAHL#AF1d0EEc@)IOS)!O)U7#hT9m!4W1,*oKK&97SV0pAs$!QN<ER56/=tp`\\QcJ6)b9#5eVn)W5bR:B>AFbG?"%F$9<c)Z34_(5uuNl^WO55h=l3"S)$nfQqGOF(2TZ5N>^5Pf$34G!7Rt<\\RmR=V]#Cb-D:&CIAnn=WO[LN(j42lrZlAf,ub[_:EGr(grmr,iDe?Puf;GJ7BKEK2><G+1.tibVDh.^p*q)rQ8,Z1J\\,Zi[:`lI_iXeH(0lH#>b1,*^8hU`PSe-^=K;daC^?6Wkqk^XMW+jZr;dj0ZVl7aOlOQ"P\'/urop@P::9Z-??K;>Q*kM,K\'mHU44.K$FXPp8KMHhm(hq-HAK1rL.GCLBEbiii;^,n"?L3`Lk7p_;)^dV%GeN0`0.23jHD6iB=0g,>75ipYK;$sLM33(536X>Ja"gARG5Z9)dsX9a_p%iA-k\'XV8[MDb,_<CfSN4GJXjcaNh8@,t8!TR+^[20GfWba(i9!f7^lKhV%[5O?PV&eIW!FnEd\\HgD0d-^\'\']eD#HAPO`<*//DqtG-Cnit`E,JWnPJHo(aS=*+b>=kR;VlgtVIWI.3)<s;V<8hG<YF!2]VnSZAhg)+?*MTdeM$itS9KElt0c43/GB`.$NO>$i^KFe>N]3Tmm5(""=A\'D+&LGs,V3cZDNqqapL6HRhrUD[j[Nq63S%!u4)=3.KDUh&,G5HQsl=]cSgs-2]m6/[[hfP:hm2\\EM,]_8XH*]P(pA(:BbM,B%9N7sOlt9`>Pq@<W@(ob98q7D>l\'Es!(SZ:"?3.:OaB!UtKc0MJ1h!FaR2F.)c1`0p5gq;s5qB[J*DqXW>?AA6H_of+1SIgGck<@tY<kaD8=`5Ti-Bbc&QFU\'0lBhr[2`h5fIt0o@[WKIs/20Wn\\_??IcU"b.kX\\(XtPZ$;*X5jeQ<T0j\'TTtA?u~>'



import unittest, sys
from rlextra.graphics.guiedit import mutils
from reportlab.lib.testutils import makeSuiteForClasses

class ImportingTestCase(unittest.TestCase):
    def test_importing_all(self):
        FAILS=[]
        M=mutils.getModules(
                    P=['reportlab.lib', 'reportlab', 'rlextra'],
                    X=[],
                    FAILS=FAILS,
                    allowCompiled=False,
                    )
        assert not FAILS,'Could not import modules\n %s' % ' \n'.join(FAILS)

def makeSuite():
    return makeSuiteForClasses(ImportingTestCase)

#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())



#Autogenerated by ReportLab guiedit do not edit
from rlextra.graphics.guiedit.datacharts import ODBCDataSource, DataAssociation, DataAwareDrawing
from reportlab.pdfbase.pdfmetrics import stringWidth
from reportlab.graphics.shapes import _DrawingEditorMixin, String
from reportlab.graphics.charts.textlabels import _simpleSplit

class DataTester(_DrawingEditorMixin,DataAwareDrawing):
	def __init__(self,width=400,height=200,*args,**kw):
		DataAwareDrawing.__init__(*(self,width,height)+args, **kw)
		self.dataSource = ODBCDataSource()
		self.dataSource.host = 'www.reportlab.co.uk'
		self.dataSource.name = 'testdb'
		self.dataSource.driver = 'mysql'
		self.dataSource.password = 'whistl3r'
		self.dataSource.user = 'rptlabi'
		self.dataSource.sql = 'SELECT chartId,periodEnd,fundReturn,indexReturn FROM testtable'
		self.dataSource.groupingColumn = 0
		self.dataSource.associations.size = 2
		self.dataSource.associations.element00 = DataAssociation(column=0, target='chartId', assocType='scalar')
		self.dataSource.associations.element01 = DataAssociation(column=((1,2),(1,3)), target='data', assocType='tmatrix')
		self.chartId = None
		self.data = None
		self.fontName = 'Helvetica'
		self.fontSize = 10
		self.leading = 1.2*self.fontSize

	def getContents(self):
		SW = lambda text, fN=self.fontName, fS=self.fontSize: stringWidth(text, fN, fS)
		L=['chartId: %s'%self.chartId] + _simpleSplit(str(self.data),self.width-20,SW)
		self.contents[:] = [String(10,self.height-self.leading*(1+i),l,fontName=self.fontName,fontSize=self.fontSize) for i,l in enumerate(L)]
		return DataAwareDrawing.getContents(*(self,))

if __name__=="__main__": #NORUNTESTS
	DataTester().go()



from functools import reduce
#Original C version by R. G. Becker 1980
def vecRound(v, s, m):
	'''
	Given the list of numbers v,
	return a list, r, such that
	sum r[i] == s

	and r = m*v with a small number of deviations
	'''
	
	n = len(v)
	if not m: return (n*[0])[:]
	vs = 0
	r = []
	for i in v:
		ir = int(i*m+0.5)
		r.append(float(ir/m))
		vs += ir
	if vs==s: return r
	rincr = 1.0/m
	incr = 1
	if vs<s:
		xs = r
		rs = v
	else:
		xs = v
		rs = r
		incr = -incr
		rincr = -rincr

	while vs!=s:
		o = 10*abs(s)			# infinity
		for i in range(n):
			t = float(xs[i]-rs[i])
			if t<=o:
				if o==t:
					if abs(v[i])>abs(v[j]): j = i
				else:
					j = i
					o = t
		vs += incr
		r[j] += rincr
	return r

def _vecRound(v, s):
	'''
	Given the list of numbers v,
	return a list, r, such that
	sum(r) == int(s+0.5) and r is close to v
	'''
	if type(s) is float: s = int(s+0.5)
	r = [int(i+0.5) for i in v]
	vs = sum(r)
	if vs==s: return r
	if vs<s:
		incr = 1
		xs = r
		rs = v
	else:
		xs = v
		rs = r
		incr = -1

	inf = 10*abs(s)
	N = range(len(v))
	while vs!=s:
		o = inf
		for i in N:
			t = xs[i]-rs[i]
			if t<=o:
				if o==t:
					if abs(v[i])>abs(v[j]): j = i
				else:
					j = i
					o = t
		vs += incr
		r[j] += incr
	return r

def dpVecFormat(v,s,dp,fmt='string'):
	m = 10**dp
	I = _vecRound([_*m for _ in v],s*m)
	if fmt=='string':
		if dp==0: return list(map(str,I))
		sfmt = '%%s.%%0%dd' % dp
		return [sfmt % divmod(int(i),m) for i in I]
	elif fmt=='value':
		if dp==0: return I
		m = float(m)
		return [i/m for i in I]
	elif fmt=='pair':
		return [divmod(int(i),m) for i in I]
	else:
		return I

def test():
	v=[9350,9350,9350,18700,39750,18700]
	import operator
	sum = float(reduce(operator.add,v))
	percen = list(map(lambda x,sum=sum: 100*x/sum, v))
	print(v)
	print(percen)
	print(vecRound(percen,100,1))
	print(vecRound(percen,100,10))
	print(vecRound(percen,100,100))
	print(dpVecFormat([18.2,48.0,33.7],100,1))
	print(dpVecFormat([1.2,5.7,2.8,8.5],18.2,1))
	print(dpVecFormat([9.0,2.7,1.8,34.5],48.1,1))
	print(dpVecFormat([6.6,12.2,9.6,5.4],33.7,1))

if __name__=='__main__':
	test()
